package com.compumark.gradle.plugin.ivy;

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;

import javax.xml.transform.Source;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.stream.StreamResult;
import javax.xml.transform.stream.StreamSource;

import org.apache.ivy.Ivy;
import org.apache.ivy.core.cache.ResolutionCacheManager;
import org.apache.ivy.core.report.ResolveReport;
import org.apache.ivy.plugins.report.XmlReportOutputter;
import org.apache.ivy.util.FileUtil;
import org.gradle.api.GradleException;
import org.gradle.api.internal.ConventionTask;
import org.gradle.api.tasks.TaskAction;

public class Depends extends ConventionTask {
	
	private Ivy ivy;
	private ResolveReport resolveReport;
	private String extension = "html";

	@TaskAction
    protected void depends() {
		ResolutionCacheManager cacheMgr = ivy.getResolutionCacheManager();
		
		File outDir = new File(getProject().getBuildDir(), "reports/ivy");
        if (!outDir.exists()) {
            if (!outDir.mkdirs()) {
                throw new GradleException("Unable to create directory: " + outDir.getAbsolutePath());
            }
        }	
        
      	InputStream xsltStream = null;
        try {
            xsltStream = new BufferedInputStream(XmlReportOutputter.class.getResourceAsStream("ivy-report.xsl"));
            Source xsltSource = new StreamSource(xsltStream);            
            TransformerFactory transformerFactory = TransformerFactory.newInstance();
            Transformer transformer = transformerFactory.newTransformer(xsltSource);	            
            transformer.setParameter("extension", extension);
    		
            for (String configuration : resolveReport.getConfigurations()) {            	
                	
                File reportFile = cacheMgr.getConfigurationResolveReportInCache(resolveReport.getResolveId(), configuration);
                File outFile = new File(outDir, configuration + "." + extension);
                
                InputStream inStream = null;
                OutputStream outStream = null;
                try {
                    inStream = new BufferedInputStream(new FileInputStream(reportFile));
                    outStream = new BufferedOutputStream(new FileOutputStream(outFile));
                    StreamResult res = new StreamResult(outStream);
                    Source src = new StreamSource(inStream);
                    transformer.transform(src, res);
                } catch (TransformerException e) {
                    throw new GradleException(e.getMessage(), e);
                } finally {
                    if (inStream != null) {
                        try {
                            inStream.close();
                        } catch (IOException e) {}
                    } 
                    if (outStream != null) {
                        try {
                            outStream.close();
                        } catch (IOException e) {}
                    }
                }
            }
        } catch (Exception e) {
        	throw new GradleException(e.getMessage(), e);
        } finally {
            if (xsltStream != null) {
                try {
                    xsltStream.close();
                } catch (IOException e) {}
            }
        }   
        
	    File css = new File(outDir, "ivy-report.css");
	    if (!css.exists()) {
	    	try {
	    		FileUtil.copy(XmlReportOutputter.class.getResourceAsStream("ivy-report.css"), css, null);
	        }
	        catch (IOException e) {
	        	throw new GradleException("Failed to copy ivy-report.css", e);	        
	        }	        
	    }           
    }

	public Ivy getIvy() {
		return ivy;
	}

	public void setIvy(Ivy ivy) {
		this.ivy = ivy;
	}

	public ResolveReport getResolveReport() {
		return resolveReport;
	}

	public void setResolveReport(ResolveReport resolveReport) {
		this.resolveReport = resolveReport;
	}

}
